"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OperationRecord = void 0;
const uuid_1 = require("uuid");
const types_1 = require("../types");
const error_1 = require("../utils/error");
const db_1 = require("../utils/db");
const datetime_handlers_1 = require("../utils/datetime-handlers");
class OperationRecord {
    constructor(obj) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        this.validateEveryNewOperation(obj);
        if (obj.type === types_1.OperationType.Payment) {
            this.validateNewPaymentOperation(obj);
        }
        this.id = (_a = obj.id) !== null && _a !== void 0 ? _a : (0, uuid_1.v4)();
        this.userId = obj.userId;
        this.periodId = (_b = obj.periodId) !== null && _b !== void 0 ? _b : null;
        this.type = obj.type;
        this.isRepetitive = Boolean(obj.isRepetitive);
        this.amount = Number(obj.amount.toFixed(2));
        this.category = (_c = obj.category) !== null && _c !== void 0 ? _c : null;
        this.subcategory = (_d = obj.subcategory) !== null && _d !== void 0 ? _d : null;
        this.description = (_e = obj.description) !== null && _e !== void 0 ? _e : null;
        this.imgUrl = (_f = obj.imgUrl) !== null && _f !== void 0 ? _f : null;
        this.lat = (_g = obj.lat) !== null && _g !== void 0 ? _g : null;
        this.lon = (_h = obj.lon) !== null && _h !== void 0 ? _h : null;
        this.createdAt = obj.createdAt ? (0, datetime_handlers_1.getStandardFormattedDateTime)(new Date(obj.createdAt)) : (0, datetime_handlers_1.getStandardFormattedDateTime)();
        this.originId = (_j = obj.originId) !== null && _j !== void 0 ? _j : null;
    }
    validateEveryNewOperation(obj) {
        if (!obj.userId || typeof obj.userId !== 'string') {
            throw new Error('User ID is required.');
        }
        if (obj.periodId && typeof obj.periodId !== 'string') {
            throw new Error('periodId should be type of string.');
        }
        if (typeof obj.isRepetitive === 'undefined') {
            throw new Error('isRepetitive is required.');
        }
        if (!obj.amount || typeof Number(obj.amount) !== 'number' || Number(obj.amount) < -999999999.99 || Number(obj.amount > 999999999.99)) {
            throw new error_1.ValidationError('Kwota operacji powinna zawierać się w przedziale od -999 999 999,99 do 999 999 999,99.');
        }
        if (obj.type < 0 || typeof obj.type !== 'number') {
            throw new error_1.ValidationError('Wybrano błędny typ kategorii operacji.');
        }
        if (obj.description && typeof obj.description !== 'string') {
            throw new error_1.ValidationError('Opis powinien być tekstem o maksymalnej długości 50 znaków.');
        }
        if (obj.description && obj.description.length > 50) {
            throw new error_1.ValidationError('Opis powinien być tekstem o maksymalnej długości 50 znaków.');
        }
        if (obj.originId && !obj.isRepetitive) {
            throw new Error('Operation with origin should be repetitive.');
        }
    }
    ;
    validateNewPaymentOperation(obj) {
        if (obj.category < 0 || typeof obj.category !== 'number') {
            throw new error_1.ValidationError('Dla każdej płatności jej kategoria jest wymagana.');
        }
        if (obj.subcategory < 0 || typeof obj.subcategory !== 'number') {
            throw new error_1.ValidationError('Dla każdej płatności jej podkategoria jest wymagana.');
        }
        if (obj.imgUrl && typeof obj.imgUrl !== 'string') {
            throw new error_1.ValidationError('Adres URL zdjęcia powinien być tekstem o długości maksymalnie 100 znaków.');
        }
        if (obj.imgUrl && obj.imgUrl.length > 100) {
            throw new error_1.ValidationError('Adres URL zdjęcia powinien być tekstem o długości maksymalnie 100 znaków.');
        }
        if (obj.lat && typeof obj.lat !== 'number') {
            throw new error_1.ValidationError('Nie można ustalić lokalizacji.');
        }
        if (obj.lon && typeof obj.lon !== 'number') {
            throw new error_1.ValidationError('Nie można ustalić lokalizacji.');
        }
    }
    ;
    static getOne(id, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `operations` WHERE id = :id AND `userId` = :userId", {
                id,
                userId
            });
            if (results.length > 1) {
                throw new Error('OperationRecord.getOne found more than 1 record.');
            }
            return results.length === 0 ? null : new OperationRecord(results[0]);
        });
    }
    ;
    static getAll(description, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `operations` WHERE `description` LIKE :search AND `userId` = :userId ORDER BY `createdAt` DESC", {
                search: `%${description}%`,
                userId,
            });
            return results.map(obj => new OperationRecord(obj));
        });
    }
    ;
    static findPeriodOperations(periodId, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `operations` WHERE `periodId` = :periodId AND `userId` = :userId ORDER BY `createdAt` DESC", {
                periodId,
                userId,
            });
            return results.map(obj => new OperationRecord(obj));
        });
    }
    ;
    static findRepetitiveOperations(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!userId) {
                throw new Error('findRepetitiveOperations requires userId.');
            }
            const [results] = yield db_1.pool.execute("SELECT * FROM `operations` WHERE `userId` = :userId AND `isRepetitive` = 1 AND `periodId` IS NULL ORDER BY `createdAt` DESC", {
                userId,
            });
            return results.map(obj => new OperationRecord(obj));
        });
    }
    ;
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("INSERT INTO `operations` (`id`, `userId`, `periodId`, `type`, `category`, `subcategory`, `description`, `isRepetitive`, `amount`, `imgUrl`, `lat`, `lon`, `createdAt`, `originId`) VALUES (:id, :userId, :periodId, :type, :category, :subcategory, :description, :isRepetitive, :amount, :imgUrl, :lat, :lon, :createdAt, :originId)", this);
            return this.id;
        });
    }
    ;
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id) {
                throw new Error('Error while deleting: given record has no ID!');
            }
            const result = yield db_1.pool.execute("DELETE FROM `operations` WHERE `id` = :id AND `userId` = :userId", {
                id: this.id,
                userId: this.userId,
            });
            if (result[0].affectedRows === 0) {
                throw new Error('Error while deleting, number of affected rows is 0.');
            }
            return true;
        });
    }
    ;
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id) {
                throw new Error('Error while updating: given record has no ID!');
            }
            const result = yield db_1.pool.execute("UPDATE `operations` SET `type` = :type, `category` = :category, `subcategory` = :subcategory, `description` = :description, `isRepetitive` = :isRepetitive, `amount` = :amount, `imgUrl` = :imgUrl, `lat` = :lat, `lon` = :lon, `originId` = :originId WHERE `id` = :id AND `userId` = :userId", this);
            if (result[0].affectedRows === 0) {
                throw new Error('Error while updating, number of affected rows is 0.');
            }
            return this.id;
        });
    }
    ;
}
exports.OperationRecord = OperationRecord;
//# sourceMappingURL=operation.record.js.map