"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PeriodRecord = void 0;
const uuid_1 = require("uuid");
const date_fns_1 = require("date-fns");
const datetime_handlers_1 = require("../utils/datetime-handlers");
const db_1 = require("../utils/db");
const error_1 = require("../utils/error");
const user_record_1 = require("./user.record");
class PeriodRecord {
    constructor(obj) {
        var _a, _b, _c, _d, _e;
        this.validateNewPeriodEntity(obj);
        this.id = (_a = obj.id) !== null && _a !== void 0 ? _a : (0, uuid_1.v4)();
        this.userId = obj.userId;
        this.isActive = Boolean(obj.isActive);
        this.starts = obj.starts ? (0, datetime_handlers_1.getStandardFormattedDateTime)(new Date(obj.starts)) : (0, datetime_handlers_1.getFirstAndLastDateOfActualMonth)().firstDateOfMonth;
        this.ends = obj.ends ? (0, datetime_handlers_1.getStandardFormattedDateTime)(new Date(obj.ends)) : (0, datetime_handlers_1.getFirstAndLastDateOfActualMonth)().lastDateOfMonth;
        this.budgetAmount = Number(((_b = obj.budgetAmount) !== null && _b !== void 0 ? _b : 0).toFixed(2));
        this.paymentsAmount = Number(((_c = obj.paymentsAmount) !== null && _c !== void 0 ? _c : 0).toFixed(2));
        this.savingsAmount = Number(((_d = obj.savingsAmount) !== null && _d !== void 0 ? _d : 0).toFixed(2));
        this.freeCashAmount = Number(((_e = obj.freeCashAmount) !== null && _e !== void 0 ? _e : 0).toFixed(2));
        this.createdAt = obj.createdAt ? (0, datetime_handlers_1.getStandardFormattedDateTime)(new Date(obj.createdAt)) : (0, datetime_handlers_1.getStandardFormattedDateTime)();
    }
    validateNewPeriodEntity(obj) {
        if (!obj.userId || typeof obj.userId !== `string`) {
            throw new Error('User ID is required.');
        }
        if (typeof obj.isActive === 'undefined') {
            throw new Error('isActive is required.');
        }
    }
    ;
    initUserBudgetAmount() {
        return __awaiter(this, void 0, void 0, function* () {
            this.budgetAmount = 5000;
            return true;
        });
    }
    ;
    static getActual(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `periods` WHERE `isActive` = 1 AND `userId` = :userId", {
                userId
            });
            if (results.length > 1) {
                throw new Error('User has active more than 1 period.');
            }
            return results.length === 0 ? null : new PeriodRecord(results[0]);
        });
    }
    ;
    static getOne(id, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `periods` WHERE id = :id AND `userId` = :userId", {
                id,
                userId
            });
            if (results.length > 1) {
                throw new Error('OperationRecord.getOne found more than 1 record.');
            }
            return results.length === 0 ? null : new PeriodRecord(results[0]);
        });
    }
    ;
    static getAll(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `periods` WHERE `userId` = :userId ORDER BY `createdAt` DESC", {
                userId,
            });
            return results.map(obj => new PeriodRecord(obj));
        });
    }
    ;
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield db_1.pool.execute("UPDATE `periods` SET `isActive` = :isActive, `budgetAmount` = :budgetAmount, `paymentsAmount` = :paymentsAmount, `savingsAmount` = :savingsAmount, `freeCashAmount` = :freeCashAmount WHERE `id` = :id AND `userId` = :userId", this);
            if (result[0].affectedRows === 0) {
                throw new Error('Error while updating, number of affected rows is 0.');
            }
        });
    }
    ;
    addPaymentOperation(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (amount >= 0) {
                throw new Error('Given amount should be smaller than 0.');
            }
            if (this.freeCashAmount < amount * -1) {
                throw new error_1.ValidationError('Kwota operacji przewyższa sumę dostępnych środków.');
            }
            this.freeCashAmount += Number(amount.toFixed(2));
            this.paymentsAmount += Number((amount * -1).toFixed(2));
            yield this.update();
        });
    }
    ;
    reversePaymentOperation(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (amount >= 0) {
                throw new Error('Given amount should be smaller than 0.');
            }
            if (this.paymentsAmount < amount * -1) {
                throw new error_1.ValidationError('Kwota operacji przewyższa sumę dostępnych środków.');
            }
            this.freeCashAmount += Number((amount * -1).toFixed(2));
            this.paymentsAmount += Number(amount.toFixed(2));
            yield this.update();
        });
    }
    addToSavingsOperation(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (amount > 0) {
                throw new Error('Given amount should be smaller than 0.');
            }
            if (this.freeCashAmount < amount * -1) {
                throw new error_1.ValidationError('Kwota operacji przewyższa sumę dostępnych środków.');
            }
            this.freeCashAmount += Number(amount.toFixed(2));
            this.savingsAmount += Number((amount * -1).toFixed(2));
            yield this.update();
        });
    }
    ;
    addFromSavingsOperation(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (amount < 0) {
                throw new Error('Given amount should be greater than 0.');
            }
            if (this.savingsAmount < amount) {
                throw new error_1.ValidationError('Kwota operacji przewyższa sumę dostępnych środków.');
            }
            this.savingsAmount += Number((amount * -1).toFixed(2));
            this.freeCashAmount += Number(amount.toFixed(2));
            yield this.update();
        });
    }
    ;
    changeBudgetOperation(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (amount < 0) {
                if (this.freeCashAmount < amount * -1 || this.budgetAmount < amount * -1) {
                    throw new error_1.ValidationError('Kwota operacji przewyższa sumę dostępnych środków.');
                }
            }
            else if (amount > 0) {
                if ((this.budgetAmount + amount) > 999999999.99) {
                    throw new error_1.ValidationError('Przekroczono maksymalną kwotę budżetu wynoszącą 999 999 999.99');
                }
            }
            else {
                throw new error_1.ValidationError('Kwota operacji nie może wynosić 0.00.');
            }
            this.freeCashAmount += Number(amount.toFixed(2));
            this.budgetAmount += Number(amount.toFixed(2));
            yield this.update();
        });
    }
    ;
    addToCushionOperation(amount, periodToClose) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield user_record_1.UserRecord.getOneById(this.userId);
            if (!user) {
                throw new Error('User not found.');
            }
            if (amount > 0) {
                throw new Error('Given amount should be smaller than 0.');
            }
            if (amount === 0)
                return;
            if ((user.financialCushion + (amount * -1)) > 999999999.99) {
                throw new error_1.ValidationError('Przekroczono maksymalną kwotę poduszki finansowej wynoszącą 999 999 999.99.');
            }
            // const savedAmount = this.savingsAmount + this.freeCashAmount;
            if (amount < 0) {
                if (!periodToClose) {
                    if (this.freeCashAmount < (amount * -1)) {
                        throw new error_1.ValidationError('Kwota operacji przewyższa sumę dostępnych środków.');
                    }
                    this.freeCashAmount += Number(amount.toFixed(2));
                    yield this.update();
                }
                user.financialCushion += amount * -1;
                yield user.update();
            }
        });
    }
    ;
    addFromCushionOperation(amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield user_record_1.UserRecord.getOneById(this.userId);
            if (!user) {
                throw new Error('User not found.');
            }
            if (amount < 0) {
                throw new Error('Given amount should be greater than 0.');
            }
            if (amount > user.financialCushion) {
                throw new error_1.ValidationError('Kwota operacji przewyższa sumę dostępnych środków.');
            }
            else {
                user.financialCushion -= amount;
                this.freeCashAmount += Number(amount.toFixed(2));
                this.budgetAmount += Number(amount.toFixed(2));
                yield this.update();
                yield user.update();
            }
        });
    }
    ;
    closePeriod() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isActive === false) {
                throw new Error('Trying to close period which is already closed.');
            }
            else if (this.isActive === true) {
                yield this.addToCushionOperation(((this.freeCashAmount + this.savingsAmount) * -1), true);
                this.isActive = false;
                yield this.update();
            }
            else {
                throw new Error('isActive should be a boolean value.');
            }
        });
    }
    ;
    checkIfActualPeriodShouldEnd() {
        const now = new Date((0, datetime_handlers_1.getStandardFormattedDateTime)());
        const datetimeWhenPeriodShouldEnd = new Date(this.ends);
        return (0, date_fns_1.isAfter)(now, datetimeWhenPeriodShouldEnd);
    }
    ;
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            const checkIfUserHasNoActivePeriods = yield PeriodRecord.getActual(this.userId);
            if (checkIfUserHasNoActivePeriods !== null) {
                throw new Error('User already has an active period. You need to close it before inserting a new one.');
            }
            yield db_1.pool.execute("INSERT INTO `periods` (`id`, `userId`, `isActive`, `starts`, `ends`, `budgetAmount`, `paymentsAmount`, `savingsAmount`, `freeCashAmount`, `createdAt`) VALUES (:id, :userId, :isActive, :starts, :ends, :budgetAmount, :paymentsAmount, :savingsAmount, :freeCashAmount, :createdAt)", this);
            return this.id;
        });
    }
    ;
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id) {
                throw new Error('Error while deleting: given record has no ID!');
            }
            const result = yield db_1.pool.execute("DELETE FROM `periods` WHERE `id` = :id AND `userId` = :userId", this);
            if (result[0].affectedRows === 0) {
                throw new Error('Error while deleting, number of affected rows is 0.');
            }
        });
    }
    ;
}
exports.PeriodRecord = PeriodRecord;
//# sourceMappingURL=period.record.js.map