"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserRecord = void 0;
const uuid_1 = require("uuid");
const db_1 = require("../utils/db");
const types_1 = require("../types");
const error_1 = require("../utils/error");
class UserRecord {
    constructor(obj) {
        var _a, _b, _c, _d, _e, _f;
        this.validateNewUserEntity(obj);
        this.id = (_a = obj.id) !== null && _a !== void 0 ? _a : (0, uuid_1.v4)();
        this.status = obj.status;
        this.confirmationCode = obj.confirmationCode;
        this.resetPwdCode = (_b = obj.resetPwdCode) !== null && _b !== void 0 ? _b : null;
        this.email = obj.email;
        this.password = obj.password;
        this.name = obj.name;
        this.financialCushion = (_c = Number(obj.financialCushion)) !== null && _c !== void 0 ? _c : 0;
        this.defaultBudgetAmount = (_d = Number(obj.defaultBudgetAmount)) !== null && _d !== void 0 ? _d : 0;
        this.localizationSource = (_e = obj.localizationSource) !== null && _e !== void 0 ? _e : types_1.LocalizationSource.None;
        this.addLocalizationByDefault = (_f = Boolean(obj.addLocalizationByDefault)) !== null && _f !== void 0 ? _f : false;
    }
    validateNewUserEntity(obj) {
        if (!obj.password || typeof obj.password !== 'string') {
            throw new error_1.ValidationError('Użytkownik musi posiadać hasło.');
        }
        if (!obj.email || typeof obj.email !== 'string') {
            throw new error_1.ValidationError('Użytkownik musi posiadać adres email.');
        }
        if (!obj.name || typeof obj.name !== `string`) {
            throw new error_1.ValidationError('Użytkownik musi posiadać imię.');
        }
        if (obj.name.length < 3 || obj.name.length > 15) {
            throw new error_1.ValidationError('Imię powinno zawierać od 3 do 15 znaków.');
        }
        if (!obj.email.match(/^[a-zA-Z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$/)) {
            throw new error_1.ValidationError('Podano nieprawidłowy adres email.');
        }
        if (obj.financialCushion && (isNaN(Number(obj.financialCushion)) || obj.financialCushion < 0 || obj.financialCushion > 999999999.99)) {
            throw new error_1.ValidationError('FinancialCushion must be a number be a number between 0 and 999 999 999.99');
        }
        if (obj.defaultBudgetAmount && (isNaN(Number(obj.defaultBudgetAmount)) || obj.defaultBudgetAmount < 0 || obj.defaultBudgetAmount > 999999.99)) {
            throw new error_1.ValidationError('DefaultBudgetAmount must be a number between 0 and 999 999.99.');
        }
        if (obj.localizationSource && !([0, 1, 2].includes(obj.localizationSource))) {
            throw new Error('Given localizationSource is invalid.');
        }
    }
    ;
    static getOneByName(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `users` WHERE `name` = :name", {
                name,
            });
            if (results.length > 1) {
                throw new Error('Found more than one user.');
            }
            return results.length === 0 ? null : new UserRecord(results[0]);
        });
    }
    ;
    static getOneByConfirmationCode(confirmationCode, isPwdReset) {
        return __awaiter(this, void 0, void 0, function* () {
            if (isPwdReset) {
                const [results] = yield db_1.pool.execute("SELECT * FROM `users` WHERE `resetPwdCode` = :resetPwdCode", {
                    resetPwdCode: confirmationCode,
                });
                if (results.length > 1) {
                    throw new Error('Found more than one user.');
                }
                return results.length === 0 ? null : new UserRecord(results[0]);
            }
            const [results] = yield db_1.pool.execute("SELECT * FROM `users` WHERE `confirmationCode` = :confirmationCode", {
                confirmationCode,
            });
            if (results.length > 1) {
                throw new Error('Found more than one user.');
            }
            return results.length === 0 ? null : new UserRecord(results[0]);
        });
    }
    ;
    static getOneByEmail(email) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `users` WHERE `email` = :email", {
                email,
            });
            if (results.length > 1) {
                throw new Error('Found more than one user.');
            }
            return results.length === 0 ? null : new UserRecord(results[0]);
        });
    }
    ;
    static getOneById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `users` WHERE `id` = :id", {
                id,
            });
            if (results.length > 1) {
                throw new Error('Found more than one user.');
            }
            return results.length === 0 ? null : new UserRecord(results[0]);
        });
    }
    ;
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("INSERT INTO `users` (`id`, `email`, `password`, `name`, `financialCushion`, `defaultBudgetAmount`, `localizationSource`, `addLocalizationByDefault`, `status`, `confirmationCode`) VALUES(:id, :email, :password, :name, :financialCushion, :defaultBudgetAmount, :localizationSource, :addLocalizationByDefault, :status, :confirmationCode)", this);
            return this.id;
        });
    }
    ;
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id) {
                throw new Error('Error while deleting: given user has no ID!');
            }
            const result = yield db_1.pool.execute("DELETE FROM `users` WHERE `id` = :id", {
                id: this.id,
            });
            if (result[0].affectedRows === 0) {
                throw new Error('Error while deleting, number of affected rows is 0.');
            }
            return true;
        });
    }
    ;
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id) {
                throw new Error('Error while updating: given user has no ID!');
            }
            const result = yield db_1.pool.execute("UPDATE `users` SET `financialCushion` = :financialCushion, `defaultBudgetAmount` = :defaultBudgetAmount, `localizationSource` = :localizationSource, `addLocalizationByDefault` = :addLocalizationByDefault, `confirmationCode` = :confirmationCode WHERE `id` = :id", this);
            if (result[0].affectedRows === 0) {
                throw new Error('Error while updating, number of affected rows is 0.');
            }
            return this;
        });
    }
    ;
    activateUser() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield db_1.pool.execute("UPDATE `users` SET `status`= :status WHERE `id` = :id", this);
            if (result[0].affectedRows === 0) {
                throw new Error('Error while updating, number of affected rows is 0.');
            }
        });
    }
    ;
    changePasswordOrResetToken() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield db_1.pool.execute("UPDATE `users` SET `password`= :password, `resetPwdCode` = :resetPwdCode WHERE `id` = :id", this);
            if (result[0].affectedRows === 0) {
                throw new Error('Error while updating, number of affected rows is 0.');
            }
        });
    }
}
exports.UserRecord = UserRecord;
//# sourceMappingURL=user.record.js.map