"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.operationRouter = void 0;
const express_1 = require("express");
const operation_record_1 = require("../records/operation.record");
const period_record_1 = require("../records/period.record");
const add_operation_to_budget_handler_1 = require("../utils/add-operation-to-budget-handler");
const reverse_operation_handler_1 = require("../utils/reverse-operation-handler");
const json_response_handlers_1 = require("../utils/json-response-handlers");
const user_directory_handler_1 = require("../utils/user-directory-handler");
const safe_join_path_1 = require("../utils/safe-join-path");
const deleting_img_handler_1 = require("../utils/deleting-img-handler");
const get_proper_value_types_from_req_handler_1 = require("../utils/get-proper-value-types-from-req-handler");
const upload_image_handler_1 = require("../utils/upload-image-handler");
const upload_1 = require("../utils/upload");
const error_1 = require("../utils/error");
const auth_jwt_1 = require("../utils/user/auth-jwt");
exports.operationRouter = (0, express_1.Router)()
    .get('/search/description?', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const operationList = yield operation_record_1.OperationRecord.getAll((_a = req.params.description) !== null && _a !== void 0 ? _a : '', req.userId);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, operationList);
}))
    .get('/get-period-operations/:periodId', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const operationList = yield operation_record_1.OperationRecord.findPeriodOperations(req.params.periodId, req.userId);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, operationList);
}))
    .get('/:id', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const operation = yield operation_record_1.OperationRecord.getOne(req.params.id, req.userId);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, operation);
}))
    .post('/', auth_jwt_1.verifyToken, upload_1.upload.single('image'), (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(req.userId);
    const imageUrl = yield (0, upload_image_handler_1.uploadImageHandler)(req, res);
    const newOperation = new operation_record_1.OperationRecord(Object.assign(Object.assign({}, (0, get_proper_value_types_from_req_handler_1.getProperValueTypesFromReqHandler)(req.body)), { userId: req.userId, periodId: actualPeriod.id, imgUrl: imageUrl !== null && imageUrl !== void 0 ? imageUrl : null }));
    yield (0, add_operation_to_budget_handler_1.AddOperationToBudgetHandler)(newOperation, actualPeriod);
    const newOperationId = yield newOperation.insert();
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, newOperationId);
}))
    .post('/repetitive-operation', auth_jwt_1.verifyToken, upload_1.upload.single('image'), (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(req.userId);
    const newRootOperation = new operation_record_1.OperationRecord(Object.assign(Object.assign({}, (0, get_proper_value_types_from_req_handler_1.getProperValueTypesFromReqHandler)(req.body)), { imgUrl: null, periodId: null, isRepetitive: true, userId: req.userId }));
    const imageUrl = yield (0, upload_image_handler_1.uploadImageHandler)(req, res);
    const newActualOperation = new operation_record_1.OperationRecord(Object.assign(Object.assign({}, newRootOperation), { lat: Number(req.body.lat), lon: Number(req.body.lon), id: null, periodId: actualPeriod.id, originId: newRootOperation.id, imgUrl: imageUrl !== null && imageUrl !== void 0 ? imageUrl : null }));
    yield (0, add_operation_to_budget_handler_1.AddOperationToBudgetHandler)(newActualOperation, actualPeriod);
    yield newRootOperation.insert();
    const newOperationId = yield newActualOperation.insert();
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, newOperationId);
}))
    .put('/:id', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(req.userId);
    const foundOperation = yield operation_record_1.OperationRecord.getOne(req.params.id, req.userId);
    if (foundOperation.periodId && foundOperation.periodId !== actualPeriod.id) {
        throw new error_1.ValidationError('Operacja nie należy do obecnego okresu.');
    }
    new operation_record_1.OperationRecord(Object.assign(Object.assign({}, foundOperation), { id: foundOperation.id, userId: req.userId, category: req.body.category, subcategory: req.body.subcategory, amount: req.body.amount, description: req.body.description }));
    if (foundOperation.periodId) {
        yield (0, reverse_operation_handler_1.ReverseOperationHandler)(foundOperation, actualPeriod);
    }
    foundOperation.category = req.body.category;
    foundOperation.subcategory = req.body.subcategory;
    foundOperation.amount = req.body.amount;
    foundOperation.description = req.body.description;
    if (foundOperation.periodId) {
        yield (0, add_operation_to_budget_handler_1.AddOperationToBudgetHandler)(foundOperation, actualPeriod);
    }
    const modifiedId = yield foundOperation.update();
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, modifiedId);
}))
    .delete('/:id/:childId?', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(req.userId);
    const foundOperation = yield operation_record_1.OperationRecord.getOne(req.params.id, req.userId);
    if (foundOperation.periodId && foundOperation.periodId !== actualPeriod.id) {
        throw new error_1.ValidationError('Operacja nie należy do obecnego okresu.');
    }
    if (!foundOperation.periodId) {
        const childOperation = yield operation_record_1.OperationRecord.getOne(req.params.childId, req.userId);
        if (!childOperation) {
            throw new Error(`Could not find child of operation schema.`);
        }
        childOperation.originId = null;
        childOperation.isRepetitive = false;
        yield childOperation.update();
    }
    else {
        yield (0, reverse_operation_handler_1.ReverseOperationHandler)(foundOperation, actualPeriod);
    }
    if (foundOperation.imgUrl) {
        const safeImgPath = (0, safe_join_path_1.safeJoinPhotoPath)(foundOperation.imgUrl, req.userId);
        yield (0, user_directory_handler_1.checkIfImageExists)(safeImgPath);
        yield (0, deleting_img_handler_1.deletingImgHandler)(safeImgPath);
    }
    const isRemoved = yield foundOperation.delete();
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, isRemoved);
}))
    .get('/image/:id', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const foundOperation = yield operation_record_1.OperationRecord.getOne(req.params.id, req.userId);
    if (!foundOperation) {
        throw new Error('Searching operation does not exist.');
    }
    if (!foundOperation.imgUrl) {
        throw new Error(`This operation does not have an image.`);
    }
    const safeImgPath = (0, safe_join_path_1.safeJoinPhotoPath)(foundOperation.imgUrl, req.userId);
    yield (0, user_directory_handler_1.checkIfImageExists)(safeImgPath);
    res.sendFile(safeImgPath);
}));
//# sourceMappingURL=operation.router.js.map