"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.periodRouter = void 0;
const express_1 = require("express");
const period_record_1 = require("../records/period.record");
const operation_record_1 = require("../records/operation.record");
const json_response_handlers_1 = require("../utils/json-response-handlers");
const insert_new_period_handler_1 = require("../utils/insert-new-period-handler");
const create_new_operation_from_repetitive_handler_1 = require("../utils/create-new-operation-from-repetitive-handler");
const auth_jwt_1 = require("../utils/user/auth-jwt");
exports.periodRouter = (0, express_1.Router)()
    .post('/create-new-operation-from-schema/:id', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(req.userId);
    const isCreated = yield (0, create_new_operation_from_repetitive_handler_1.createNewOperationFromRepetitiveHandler)(req.params.id, req.userId, actualPeriod);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, isCreated);
}))
    .get('/', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const periodList = yield period_record_1.PeriodRecord.getAll(req.userId);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, periodList);
}))
    .get('/actual', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(req.userId);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, actualPeriod);
}))
    .get('/:id', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const foundPeriod = yield period_record_1.PeriodRecord.getOne(req.params.id, req.userId);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, foundPeriod);
}))
    .post('/', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(req.userId);
    if (actualPeriod) {
        if (actualPeriod.checkIfActualPeriodShouldEnd()) {
            yield actualPeriod.closePeriod();
            const RepetitiveOperations = yield (0, insert_new_period_handler_1.insertNewPeriodHandler)(req.userId);
            (0, json_response_handlers_1.sendSuccessJsonHandler)(res, RepetitiveOperations);
            return;
        }
        else {
            (0, json_response_handlers_1.sendErrorJsonHandler)(res, 'Aktualny okres jeszcze się nie zakończył.');
            return;
        }
    }
    const newPeriod = yield (0, insert_new_period_handler_1.insertNewPeriodHandler)(req.userId, true);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, newPeriod);
}))
    .delete('/:id', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const foundPeriod = yield period_record_1.PeriodRecord.getOne(req.params.id, req.userId);
    const periodOperations = yield operation_record_1.OperationRecord.findPeriodOperations(foundPeriod.id, req.userId);
    for (const operation of periodOperations) {
        yield operation.delete();
    }
    yield foundPeriod.delete();
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, true);
}));
//# sourceMappingURL=period.router.js.map