"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sessionRouter = void 0;
const express_1 = require("express");
const user_record_1 = require("../records/user.record");
const error_1 = require("../utils/error");
const _auth_1 = require("../types/_auth/_auth");
const bcryptjs_1 = __importDefault(require("bcryptjs"));
const period_record_1 = require("../records/period.record");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const config_1 = require("../config/config");
const json_response_handlers_1 = require("../utils/json-response-handlers");
const auth_jwt_1 = require("../utils/user/auth-jwt");
exports.sessionRouter = (0, express_1.Router)()
    .post('/', (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const user = yield user_record_1.UserRecord.getOneByEmail(req.body.email);
    if (!user) {
        throw new error_1.ValidationError('Użytkownik lub hasło są nieprawidłowe.');
    }
    if (user.status === _auth_1.UserStatus.Pending) {
        throw new error_1.ValidationError('Konto oczekuje na aktywację poprzez link aktywacyjny wysłany na podany adres e-mail.');
    }
    const passwordIsValid = bcryptjs_1.default.compareSync(req.body.password, user.password);
    if (!passwordIsValid) {
        throw new error_1.ValidationError('Email lub hasło są nieprawidłowe.');
    }
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(user.id);
    const userToken = jsonwebtoken_1.default.sign({ id: user.id }, config_1.config.jwtSecret, {
        expiresIn: 3600,
    });
    req.session.token = userToken;
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, {
        userToken,
        user: Object.assign(Object.assign({}, user), { password: '', confirmationCode: '', resetPwdCode: '' }),
        actualPeriod: actualPeriod !== null && actualPeriod !== void 0 ? actualPeriod : null,
    });
}))
    .get('/check-user', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const currentUser = yield user_record_1.UserRecord.getOneById((req.userId));
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(currentUser.id);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, {
        user: Object.assign(Object.assign({}, currentUser), { password: '', confirmationCode: '', resetPwdCode: '' }),
        actualPeriod: actualPeriod !== null && actualPeriod !== void 0 ? actualPeriod : null,
    });
}))
    .delete('/', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    req.session = null;
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, "Wylogowano użytkownika.");
}));
//# sourceMappingURL=session.router.js.map