"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.userRouter = void 0;
const bcryptjs_1 = __importDefault(require("bcryptjs"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const express_1 = require("express");
const user_record_1 = require("../records/user.record");
const json_response_handlers_1 = require("../utils/json-response-handlers");
const error_1 = require("../utils/error");
const config_1 = require("../config/config");
const verify_sign_up_1 = require("../utils/user/verify-sign-up");
const auth_jwt_1 = require("../utils/user/auth-jwt");
const types_1 = require("../types");
const _auth_1 = require("../types/_auth/_auth");
const nodemailer_config_1 = require("../config/nodemailer.config");
exports.userRouter = (0, express_1.Router)()
    .post('/signup', verify_sign_up_1.checkDuplicateUsernameOrEmail, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    if (req.body.password !== req.body.confirmPassword) {
        throw new error_1.ValidationError('Podane hasła nie są jednakowe.');
    }
    const pwd = req.body.password;
    if (!pwd.match(/^(?=.*[0-9])(?=.*[!@#$%^&*])[a-zA-Z0-9!@#$%^&*]{7,15}$/)) {
        throw new error_1.ValidationError('Hasło powinno składać się z 7-15 znaków, w tym przynajmniej z 1 cyfry oraz znaku specjalnego.');
    }
    const emailToken = jsonwebtoken_1.default.sign({ email: req.body.email }, config_1.config.jwtSecret);
    const newUser = new user_record_1.UserRecord({
        id: null,
        name: req.body.name,
        email: req.body.email,
        password: bcryptjs_1.default.hashSync(req.body.password, 10),
        addLocalizationByDefault: false,
        defaultBudgetAmount: 0,
        financialCushion: 0,
        localizationSource: types_1.LocalizationSource.None,
        confirmationCode: emailToken,
        status: _auth_1.UserStatus.Pending,
        resetPwdCode: null,
    });
    yield newUser.insert();
    (0, nodemailer_config_1.sendConfirmationEmail)(newUser.email, "Please confirm your account", "Email Confirmation", "Welcome to MyRealCoins", "Thank you for being with us. Please confirm your email by clicking on the following link:", `confirm/${emailToken}`);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, 'Link aktywacyjny został wysłany na podany adres email. Uwaga: email mógł trafić do spamu.');
}))
    .post('/reset', (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const foundUser = yield user_record_1.UserRecord.getOneByEmail(req.body.email);
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, 'Jeśli podany email istnieje w bazie, to został wysłany link do zmiany hasła, Będzie on aktywny przez 3 godziny.');
    if (foundUser) {
        const emailToken = jsonwebtoken_1.default.sign({ email: req.body.email }, config_1.config.jwtSecret, {
            expiresIn: 10800,
        });
        foundUser.resetPwdCode = emailToken;
        yield foundUser.changePasswordOrResetToken();
        (0, nodemailer_config_1.sendConfirmationEmail)(foundUser.email, "Password reset", "Email Confirmation", "Hello!", "Thank you for being with us. To reset your password, please click on the link below. Link is valid for three hours.", `reset/${foundUser.id}/${emailToken}`);
    }
}))
    .post('/reset/:userId/:confirmationCode', (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const foundUser = yield user_record_1.UserRecord.getOneByConfirmationCode(req.params.confirmationCode, true);
    if (!foundUser || foundUser.id !== req.params.userId) {
        throw new error_1.ValidationError('Podano nieprawidłowy token.');
    }
    if (req.body.password !== req.body.confirmPassword) {
        throw new error_1.ValidationError('Podane hasła nie są jednakowe.');
    }
    const pwd = req.body.password;
    if (!pwd.match(/^(?=.*[0-9])(?=.*[!@#$%^&*])[a-zA-Z0-9!@#$%^&*]{7,15}$/)) {
        throw new error_1.ValidationError('Hasło powinno składać się z 7-15 znaków, w tym przynajmniej z 1 cyfry oraz znaku specjalnego.');
    }
    foundUser.password = bcryptjs_1.default.hashSync(req.body.password, 10);
    foundUser.resetPwdCode = null;
    yield foundUser.changePasswordOrResetToken();
    (0, nodemailer_config_1.sendConfirmationEmail)(foundUser.email, "Your password has been changed", "Your password has been changed", "Hello!", "Thank you for being with us. Your password has been changed successfully. If this is a mistake, please contact us.");
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, 'Hasło zostało zmienione. Zaloguj się.');
}))
    .get('/confirm/:confirmationCode', (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const foundUser = yield user_record_1.UserRecord.getOneByConfirmationCode(req.params.confirmationCode, false);
    if (!foundUser) {
        throw new error_1.ValidationError('Podano nieprawidłowy kod aktywacyjny.');
    }
    if (foundUser.status === _auth_1.UserStatus.Active) {
        throw new error_1.ValidationError('Konto zostało już aktywowane.');
    }
    foundUser.status = _auth_1.UserStatus.Active;
    yield foundUser.activateUser();
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, 'Konto zostało aktywowane. Zaloguj się.');
}))
    .put('/', auth_jwt_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const user = yield user_record_1.UserRecord.getOneById(req.userId);
    if (!user) {
        throw new Error('User not found!');
    }
    if (req.body.defaultBudgetAmount > 999999.99) {
        throw new error_1.ValidationError('Domyślna kwota miesięcznego budżetu nie może przekraczać 999 999.99 PLN.');
    }
    if (req.body.defaultBudgetAmount > 999999999.99) {
        throw new error_1.ValidationError('Kwota aktualnej poduszki finansowej nie może przekraczać 999 999 999.99 PLN.');
    }
    user.financialCushion = req.body.financialCushion;
    user.defaultBudgetAmount = req.body.defaultBudgetAmount;
    user.addLocalizationByDefault = req.body.addLocalizationByDefault;
    const updatedUser = yield user.update();
    (0, json_response_handlers_1.sendSuccessJsonHandler)(res, updatedUser);
}));
//# sourceMappingURL=user.router.js.map