"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const period_record_1 = require("../../records/period.record");
const types_1 = require("../../types");
const operation_record_1 = require("../../records/operation.record");
const add_operation_to_budget_handler_1 = require("../../utils/add-operation-to-budget-handler");
const db_1 = require("../../utils/db");
const firstTestPeriod = new period_record_1.PeriodRecord({
    id: "[TEST]5",
    userId: "testUserId5",
    isActive: true,
    starts: "2023-02-01 00:00:00",
    ends: "2023-02-28 23:59:59",
    budgetAmount: 6000,
    paymentsAmount: 3000,
    savingsAmount: 2000,
    freeCashAmount: 1000,
    createdAt: "2023-02-27 12:48:32",
});
const defaultOperationRecord = new operation_record_1.OperationRecord({
    userId: "test-user-id5",
    periodId: "test-period-id5",
    category: types_1.PaymentCategory.BasicNeeds,
    subcategory: types_1.BasicNeeds.Supermarket,
    description: "user-description",
    type: types_1.OperationType.Payment,
    amount: -20,
    imgUrl: "/test.jpg",
    isRepetitive: false,
    lat: 49.34342,
    lon: 49.34342,
});
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield firstTestPeriod.insert();
    yield defaultOperationRecord.insert();
}));
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield defaultOperationRecord.delete();
    yield firstTestPeriod.delete();
    yield db_1.pool.end();
}));
test(`AddOperationToBudgetHandler can modify actual user's period record and update the database.`, () => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(firstTestPeriod.userId);
    const newOperation = new operation_record_1.OperationRecord(Object.assign({}, defaultOperationRecord));
    const actualPeriodToCompare = new period_record_1.PeriodRecord(Object.assign({}, actualPeriod));
    yield (0, add_operation_to_budget_handler_1.AddOperationToBudgetHandler)(newOperation, actualPeriod);
    expect(actualPeriod.freeCashAmount).toBe(actualPeriodToCompare.freeCashAmount + defaultOperationRecord.amount);
    expect(actualPeriod.paymentsAmount).toBe(actualPeriodToCompare.paymentsAmount - defaultOperationRecord.amount);
}));
test(`AddOperationToBudgetHandler can modify actual user's period record depending on operation's type and update the database.`, () => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(firstTestPeriod.userId);
    const newOperation = new operation_record_1.OperationRecord(Object.assign(Object.assign({}, defaultOperationRecord), { type: types_1.OperationType.AddToSavings, amount: -50 }));
    const actualPeriodToCompare = new period_record_1.PeriodRecord(Object.assign({}, actualPeriod));
    yield (0, add_operation_to_budget_handler_1.AddOperationToBudgetHandler)(newOperation, actualPeriod);
    expect(actualPeriod.freeCashAmount).toBe(actualPeriodToCompare.freeCashAmount + newOperation.amount);
    expect(actualPeriod.savingsAmount).toBe(actualPeriodToCompare.savingsAmount - newOperation.amount);
}));
//# sourceMappingURL=add-operation-to-budget-handler.test.js.map