"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const period_record_1 = require("../../records/period.record");
const operation_record_1 = require("../../records/operation.record");
const reverse_operation_handler_1 = require("../../utils/reverse-operation-handler");
const types_1 = require("../../types");
const db_1 = require("../../utils/db");
const defaultOperationRecord = new operation_record_1.OperationRecord({
    userId: "test-user-id6",
    periodId: "test-period-id6",
    category: types_1.PaymentCategory.BasicNeeds,
    subcategory: types_1.BasicNeeds.Supermarket,
    description: "user-description",
    type: types_1.OperationType.Payment,
    amount: -20,
    imgUrl: "/test.jpg",
    isRepetitive: false,
    lat: 49.34342,
    lon: 49.34342,
});
const firstTestPeriod = new period_record_1.PeriodRecord({
    id: "[TEST2]6",
    userId: "testUserId6",
    isActive: true,
    starts: "2023-02-01 00:00:00",
    ends: "2023-02-28 23:59:59",
    budgetAmount: 6000,
    paymentsAmount: 3000,
    savingsAmount: 2000,
    freeCashAmount: 1000,
    createdAt: "2023-02-27 12:48:32",
});
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield firstTestPeriod.insert();
    yield defaultOperationRecord.insert();
}));
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield defaultOperationRecord.delete();
    yield firstTestPeriod.delete();
    yield db_1.pool.end();
}));
test('ReverseOperationHandler can reverse operations.', () => __awaiter(void 0, void 0, void 0, function* () {
    const actualPeriod = yield period_record_1.PeriodRecord.getActual(firstTestPeriod.userId);
    const foundOperation = yield operation_record_1.OperationRecord.getOne(defaultOperationRecord.id, defaultOperationRecord.userId);
    const foundOperationAmount = foundOperation.amount;
    const { freeCashAmount, paymentsAmount } = actualPeriod;
    yield (0, reverse_operation_handler_1.ReverseOperationHandler)(foundOperation, actualPeriod);
    const modifiedActualPeriod = yield period_record_1.PeriodRecord.getActual(firstTestPeriod.userId);
    expect(freeCashAmount).toBe(modifiedActualPeriod.freeCashAmount + foundOperationAmount);
    expect(paymentsAmount).toBe(modifiedActualPeriod.paymentsAmount - foundOperationAmount);
}));
//# sourceMappingURL=reverse-operation-handler.test.js.map