"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const period_record_1 = require("../../records/period.record");
const datetime_handlers_1 = require("../../utils/datetime-handlers");
jest
    .spyOn(period_record_1.PeriodRecord.prototype, 'initUserBudgetAmount')
    .mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () {
    return true;
}));
const defaultPeriodObject = {
    userId: 'test-userId',
    isActive: true,
    budgetAmount: 5000,
    paymentsAmount: 3000,
    savingsAmount: 1000,
    freeCashAmount: 1000,
    createdAt: '2023-02-01 00:00:00',
};
test('PeriodRecord can build proper objects.', () => {
    const per = new period_record_1.PeriodRecord(defaultPeriodObject);
    expect(per.userId).toBe('test-userId');
    expect(per.isActive).toBe(true);
    expect(per.starts).toBe((0, datetime_handlers_1.getFirstAndLastDateOfActualMonth)().firstDateOfMonth);
    expect(per.ends).toBe((0, datetime_handlers_1.getFirstAndLastDateOfActualMonth)().lastDateOfMonth);
    expect(per.budgetAmount).toBe(5000);
    expect(per.paymentsAmount).toBe(3000);
    expect(per.savingsAmount).toBe(1000);
    expect(per.freeCashAmount).toBe(1000);
    expect(per.createdAt).toBe('2023-02-01 00:00:00');
});
test('PeriodRecord creates a valid UUID for a new entry.', () => {
    const op = new period_record_1.PeriodRecord(defaultPeriodObject);
    expect(op.id).toBeDefined();
    expect(op.id).toMatch(/^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/);
});
test('PeriodRecord validates if userId exists', () => {
    expect(() => new period_record_1.PeriodRecord({
        isActive: true,
        budgetAmount: 5000,
        paymentsAmount: 3000,
        savingsAmount: 1000,
        freeCashAmount: 1000,
        createdAt: '2023-02-01 00:00:00',
    })).toThrow('User ID is required.');
});
test('PeriodRecord validates if isActive exists.', () => {
    expect(() => new period_record_1.PeriodRecord({
        userId: 'test-userId',
        budgetAmount: 5000,
        paymentsAmount: 3000,
        savingsAmount: 1000,
        freeCashAmount: 1000,
        createdAt: '2023-02-01 00:00:00',
    })).toThrow('isActive is required.');
});
test('PeriodRecord.initUserBudgetAmount can get user budget amount.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(defaultPeriodObject);
    expect(yield per.initUserBudgetAmount()).toBe(true);
}));
//# sourceMappingURL=period-record.test.js.map