"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const date_fns_1 = require("date-fns");
const db_1 = require("../../utils/db");
const period_record_1 = require("../../records/period.record");
const datetime_handlers_1 = require("../../utils/datetime-handlers");
const firstTestPeriod = new period_record_1.PeriodRecord({
    id: "[TEST]",
    userId: "testUserId",
    isActive: false,
    starts: "2023-02-01 00:00:00",
    ends: "2023-02-28 23:59:59",
    budgetAmount: 6000,
    paymentsAmount: 3000,
    savingsAmount: 2000,
    freeCashAmount: 1000,
    createdAt: "2023-02-27 12:48:32",
});
const secondTestPeriod = new period_record_1.PeriodRecord({
    id: "[TEST]2",
    userId: "testUserId",
    isActive: true,
    starts: "2023-02-01 00:00:00",
    ends: "2023-02-28 23:59:59",
    budgetAmount: 6000,
    paymentsAmount: 3000,
    savingsAmount: 2000,
    freeCashAmount: 1000,
    createdAt: "2023-02-27 12:48:32",
});
const defaultFourthPeriodObj = {
    userId: "[TEST]4",
    budgetAmount: 500,
    freeCashAmount: 100,
    isActive: false,
    paymentsAmount: 100,
    savingsAmount: 100
};
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield firstTestPeriod.insert();
    yield secondTestPeriod.insert();
}));
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield firstTestPeriod.delete();
    yield secondTestPeriod.delete();
    const thirdPeriodObj = yield period_record_1.PeriodRecord.getOne('[TEST]3', '[TEST]3');
    if (thirdPeriodObj) {
        yield thirdPeriodObj.delete();
    }
    const fourthPeriodObj = yield period_record_1.PeriodRecord.getOne('[TEST]4', '[TEST]4');
    if (thirdPeriodObj) {
        yield fourthPeriodObj.delete();
    }
    yield db_1.pool.end();
}));
test('PeriodRecord.getOne returns data from database for one entry.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]', 'testUserId');
    expect(per).toBeDefined();
    expect(per.id).toBe('[TEST]');
    expect(per.userId).toBe('testUserId');
    expect(per.isActive).toBe(false);
    expect(per.starts).toBe('2023-02-01 00:00:00');
    expect(per.ends).toBe('2023-02-28 23:59:59');
    expect(per.budgetAmount).toBe(6000.00);
    expect(per.paymentsAmount).toBe(3000.00);
    expect(per.savingsAmount).toBe(2000.00);
    expect(per.freeCashAmount).toBe(1000.00);
}));
test('PeriodRecord.getOne returns null for non-existing entry.', () => __awaiter(void 0, void 0, void 0, function* () {
    const op = yield period_record_1.PeriodRecord.getOne('[non-existing-entry]', 'testUserId');
    expect(op).toBeNull();
}));
test('PeriodRecord.getOne returns null when operation ID is valid, but userID is wrong.', () => __awaiter(void 0, void 0, void 0, function* () {
    const op = yield period_record_1.PeriodRecord.getOne('[TEST]2', '[non-existing-userId]');
    expect(op).toBeNull();
}));
test('PeriodRecord.getAll returns an array of found entries.', () => __awaiter(void 0, void 0, void 0, function* () {
    const opArray = yield period_record_1.PeriodRecord.getAll('testUserId');
    expect(opArray).not.toEqual([]);
    expect(opArray[0].id).toBeDefined();
}));
test('PeriodRecord.getAll returns empty array of found entries when userId is wrong.', () => __awaiter(void 0, void 0, void 0, function* () {
    const opArray = yield period_record_1.PeriodRecord.getAll('[non-existing-userId]');
    expect(opArray).toEqual([]);
}));
test('PeriodRecord.changeBudgetOperation increases budgetAmount and freeCashAmount when amount is greater than 0 and updates the database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    const predictedBudgetAmount = per.budgetAmount + 200;
    const predictedFreeCashAmount = per.freeCashAmount + 200;
    yield per.changeBudgetOperation(200);
    const updatedPer = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    expect(updatedPer.budgetAmount).toBe(predictedBudgetAmount);
    expect(updatedPer.freeCashAmount).toBe(predictedFreeCashAmount);
}));
test('PeriodRecord.changeBudgetOperation reduces budgetAmount and freeCashAmount when amount is smaller than 0 and updates the database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    const predictedBudgetAmount = per.budgetAmount - 1;
    const predictedFreeCashAmount = per.freeCashAmount - 1;
    yield per.changeBudgetOperation(-1);
    const updatedPer = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    expect(updatedPer.budgetAmount).toBe(predictedBudgetAmount);
    expect(updatedPer.freeCashAmount).toBe(predictedFreeCashAmount);
}));
test('PeriodRecord.changeBudgetOperation throws when given amount is greater than 999999999.99, when amount is greater than freeCashAmount when reducing and when amount = 0.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.changeBudgetOperation(9999999999999999999); })).rejects.toThrow('Przekroczono maksymalną kwotę budżetu wynoszącą 999 999 999.99');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.changeBudgetOperation(-99999999999); })).rejects.toThrow('Kwota operacji przewyższa sumę dostępnych środków.');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.changeBudgetOperation(0); })).rejects.toThrow('Kwota operacji nie może wynosić 0.00.');
}));
test('PeriodRecord.addPaymentOperation reduces freeCashAmount and increases paymentsAmount.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    const predictedFreeCashAmount = per.freeCashAmount - 1;
    const predictedPaymentsAmount = per.paymentsAmount + 1;
    yield per.addPaymentOperation(-1);
    const updatedPer = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    expect(updatedPer.freeCashAmount).toBe(predictedFreeCashAmount);
    expect(updatedPer.paymentsAmount).toBe(predictedPaymentsAmount);
}));
test('PeriodRecord.addPaymentOperation throws when given amount is greater than freeCashAmount or is incorrect.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.addPaymentOperation(1); })).rejects.toThrow('Given amount should be smaller than 0.');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.addPaymentOperation(-99999999999); })).rejects.toThrow('Kwota operacji przewyższa sumę dostępnych środków.');
}));
test('PeriodRecord.addToSavingsOperation reduces freeCashAmount and increases savingsAmount.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    const predictedFreeCashAmount = per.freeCashAmount - 1;
    const predictedSavingsAmount = per.savingsAmount + 1;
    yield per.addToSavingsOperation(-1);
    const updatedPer = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    expect(updatedPer.freeCashAmount).toBe(predictedFreeCashAmount);
    expect(updatedPer.savingsAmount).toBe(predictedSavingsAmount);
}));
test('PeriodRecord.addToSavingsOperation throws when given amount is greater than freeCashAmount or is incorrect.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.addToSavingsOperation(1); })).rejects.toThrow('Given amount should be smaller than 0.');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.addToSavingsOperation(-99999999999); })).rejects.toThrow('Kwota operacji przewyższa sumę dostępnych środków.');
}));
test('PeriodRecord.addFromSavingsOperation reduces savingsAmount and increases freeCashAmount.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    const predictedFreeCashAmount = per.freeCashAmount + 1;
    const predictedSavingsAmount = per.savingsAmount - 1;
    yield per.addFromSavingsOperation(1);
    const updatedPer = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    expect(updatedPer.freeCashAmount).toBe(predictedFreeCashAmount);
    expect(updatedPer.savingsAmount).toBe(predictedSavingsAmount);
}));
test('PeriodRecord.addFromSavingsOperation throws when given amount is greater than freeCashAmount or is incorrect.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getOne('[TEST]2', 'testUserId');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.addFromSavingsOperation(-1); })).rejects.toThrow('Given amount should be greater than 0.');
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.addFromSavingsOperation(99999999999); })).rejects.toThrow('Kwota operacji przewyższa sumę dostępnych środków.');
}));
test(`PeriodRecord.getActual returns user's active period.`, () => __awaiter(void 0, void 0, void 0, function* () {
    const actualPer = yield period_record_1.PeriodRecord.getActual('testUserId');
    expect(actualPer).toBeDefined();
    expect(actualPer.id).toBe('[TEST]2');
    expect(actualPer.userId).toBe('testUserId');
    expect(actualPer.isActive).toBe(true);
    expect(actualPer.starts).toBe('2023-02-01 00:00:00');
    expect(actualPer.ends).toBe('2023-02-28 23:59:59');
}));
test(`PeriodRecord.getActual returns null when found 0 records in database.`, () => __awaiter(void 0, void 0, void 0, function* () {
    const per = yield period_record_1.PeriodRecord.getActual('[non-existing-userId]');
    expect(per).toBeNull();
}));
test('PeriodRecord.checkIfActualPeriodShouldEnd returns true when period should end.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(Object.assign(Object.assign({}, defaultFourthPeriodObj), { ends: '2023-01-31 23:59:59' }));
    const checkIfPerShouldEnd = per.checkIfActualPeriodShouldEnd();
    expect(checkIfPerShouldEnd).toBe(true);
}));
test('PeriodRecord.checkIfActualPeriodShouldEnd returns false when period.ends is later than now.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(Object.assign(Object.assign({}, defaultFourthPeriodObj), { ends: (0, datetime_handlers_1.getStandardFormattedDateTime)((0, date_fns_1.add)(new Date(), {
            days: 1,
        })) }));
    const checkIfPerShouldEnd = per.checkIfActualPeriodShouldEnd();
    expect(checkIfPerShouldEnd).toBe(false);
}));
test('PeriodRecord.insert inserts into database and returns an ID.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(Object.assign(Object.assign({}, defaultFourthPeriodObj), { userId: '[TEST]3', id: '[TEST]3', isActive: true }));
    const perId = yield per.insert();
    expect(perId).toBe(per.id);
    const foundedPer = yield period_record_1.PeriodRecord.getOne(per.id, per.userId);
    expect(foundedPer).toBeDefined();
}));
test('PeriodRecord.insert throws if user has an active period.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(Object.assign(Object.assign({}, defaultFourthPeriodObj), { userId: '[TEST]3', isActive: true }));
    yield expect(() => __awaiter(void 0, void 0, void 0, function* () { return yield per.insert(); })).rejects.toThrow('User already has an active period. You need to close it before inserting a new one.');
}));
test('PeriodRecord.closePeriod changes isActive to false and updates the database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(Object.assign(Object.assign({}, defaultFourthPeriodObj), { id: '[TEST]4', isActive: true }));
    yield per.insert();
    expect(per.isActive).toBe(true);
    yield per.closePeriod();
    expect(per.isActive).toBe(false);
    const updatedPer = yield period_record_1.PeriodRecord.getOne(per.id, per.userId);
    expect(updatedPer.isActive).toBe(false);
}));
test('PeriodRecord.delete removes record from database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(Object.assign(Object.assign({}, defaultFourthPeriodObj), { userId: '[TEST]4' }));
    yield per.insert();
    const found = yield period_record_1.PeriodRecord.getOne(per.id, per.userId);
    expect(found).toBeDefined();
    yield per.delete();
    const foundAfterDelete = yield period_record_1.PeriodRecord.getOne(per.id, per.userId);
    expect(foundAfterDelete).toBeNull();
}));
test('PeriodRecord.reversePaymentOperation can reverse effects of given operation.', () => __awaiter(void 0, void 0, void 0, function* () {
    const per = new period_record_1.PeriodRecord(Object.assign(Object.assign({}, defaultFourthPeriodObj), { userId: '[TEST]5' }));
    const perToCompare = new period_record_1.PeriodRecord(Object.assign({}, per));
    const paymentAmount = -1;
    yield per.insert();
    yield per.addPaymentOperation(paymentAmount);
    const perAfterPayment = yield period_record_1.PeriodRecord.getOne(per.id, per.userId);
    expect(perAfterPayment.freeCashAmount).toBe(perToCompare.freeCashAmount + paymentAmount);
    expect(perAfterPayment.paymentsAmount).toBe(perToCompare.paymentsAmount - paymentAmount);
    yield perAfterPayment.reversePaymentOperation(paymentAmount);
    const reversedPer = yield period_record_1.PeriodRecord.getOne(perAfterPayment.id, perAfterPayment.userId);
    expect(reversedPer.freeCashAmount).toBe(perToCompare.freeCashAmount);
    expect(reversedPer.paymentsAmount).toBe(perToCompare.paymentsAmount);
    yield reversedPer.delete();
}));
//# sourceMappingURL=period.test.js.map