"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const user_record_1 = require("../../records/user.record");
const types_1 = require("../../types");
const _auth_1 = require("../../types/_auth/_auth");
const defaultObj = {
    addLocalizationByDefault: true,
    defaultBudgetAmount: 5000,
    email: "test@example.com",
    financialCushion: 0,
    id: null,
    localizationSource: types_1.LocalizationSource.Receipt,
    name: "Test",
    password: "test-password",
    resetPwdCode: '',
    confirmationCode: '',
    status: _auth_1.UserStatus.Pending,
};
test('UserRecord can build a proper User Object.', () => {
    const newUser = new user_record_1.UserRecord(defaultObj);
    expect(newUser.id).toMatch(/^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/);
    expect(newUser.addLocalizationByDefault).toBe(true);
    expect(newUser.defaultBudgetAmount).toBe(5000);
    expect(newUser.email).toBe("test@example.com");
    expect(newUser.financialCushion).toBe(0);
    expect(newUser.localizationSource).toBe(types_1.LocalizationSource.Receipt);
    expect(newUser.name).toBe("Test");
    expect(newUser.password).toBe("test-password");
});
test('UserRecord validates if name exists and it`s length fits 3 - 15 characters.', () => {
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { name: null }))).toThrow('Użytkownik musi posiadać imię.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { name: 'a' }))).toThrow('Imię powinno zawierać od 3 do 15 znaków.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { name: 'qweqweqeqeqweqweqeqeqw54545r4eq' }))).toThrow('Imię powinno zawierać od 3 do 15 znaków.');
});
test('UserRecord validates if password is given.', () => {
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { password: null }))).toThrow('Użytkownik musi posiadać hasło.');
});
test('UserRecord validates if email exists and is correct.', () => {
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { email: null }))).toThrow('Użytkownik musi posiadać adres email.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { email: 'test' }))).toThrow('Podano nieprawidłowy adres email.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { email: 'test@' }))).toThrow('Podano nieprawidłowy adres email.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { email: 'test@example.' }))).toThrow('Podano nieprawidłowy adres email.');
});
test('UserRecord validates if financialCushion is valid.', () => {
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { financialCushion: '32wss' }))).toThrow('FinancialCushion must be a number be a number between 0 and 999 999 999.99');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { financialCushion: -1 }))).toThrow('FinancialCushion must be a number be a number between 0 and 999 999 999.99');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { financialCushion: 99999999999.99 }))).toThrow('FinancialCushion must be a number be a number between 0 and 999 999 999.99');
});
test('UserRecord validates if defaultBudgetAmount is valid.', () => {
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { defaultBudgetAmount: 'red' }))).toThrow('DefaultBudgetAmount must be a number between 0 and 999 999.99.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { defaultBudgetAmount: -1 }))).toThrow('DefaultBudgetAmount must be a number between 0 and 999 999.99.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { defaultBudgetAmount: 99999999999.99 }))).toThrow('DefaultBudgetAmount must be a number between 0 and 999 999.99.');
});
test('UserRecord validates if given localizationSource is valid.', () => {
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { localizationSource: 'red' }))).toThrow('Given localizationSource is invalid.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { localizationSource: -1 }))).toThrow('Given localizationSource is invalid.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { localizationSource: -1 }))).toThrow('Given localizationSource is invalid.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { localizationSource: 1.5 }))).toThrow('Given localizationSource is invalid.');
    expect(() => new user_record_1.UserRecord(Object.assign(Object.assign({}, defaultObj), { localizationSource: 4343 }))).toThrow('Given localizationSource is invalid.');
});
//# sourceMappingURL=user-record.test.js.map