"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../../types");
const user_record_1 = require("../../records/user.record");
const db_1 = require("../../utils/db");
const _auth_1 = require("../../types/_auth/_auth");
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield db_1.pool.end();
}));
const defaultObj = {
    addLocalizationByDefault: true,
    defaultBudgetAmount: 5000,
    email: "test@example.com",
    financialCushion: 1000,
    localizationSource: types_1.LocalizationSource.Receipt,
    name: "Test",
    password: "test-password",
    status: _auth_1.UserStatus.Pending,
    resetPwdCode: '',
    confirmationCode: '',
};
test('UserRecord.insert inserts record into database. GetOneById can find this.', () => __awaiter(void 0, void 0, void 0, function* () {
    const newUser = new user_record_1.UserRecord(defaultObj);
    const insertedId = yield newUser.insert();
    const found = yield user_record_1.UserRecord.getOneById(insertedId);
    expect(insertedId).toBe(newUser.id);
    expect(found).toBeDefined();
}));
test('UserRecord.update updates record in the database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const foundByMail = yield user_record_1.UserRecord.getOneByEmail(defaultObj.email);
    foundByMail.financialCushion = 2000;
    yield foundByMail.update();
    const foundModified = yield user_record_1.UserRecord.getOneByEmail(defaultObj.email);
    expect(foundModified.financialCushion).toBe(foundByMail.financialCushion);
}));
test('UserRecord.delete removes user from database.', () => __awaiter(void 0, void 0, void 0, function* () {
    const foundByMail = yield user_record_1.UserRecord.getOneByEmail(defaultObj.email);
    const isRemoved = yield foundByMail.delete();
    const founded = yield user_record_1.UserRecord.getOneByEmail(defaultObj.email);
    expect(isRemoved).toBe(true);
    expect(founded).toBe(null);
}));
//# sourceMappingURL=user.test.js.map